;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x2015aa424934be5d8fca000c39c83d77 . "./code.cg2js/cg-status-bar.fasl")
(#x78b9e63a7bcbeaa190f8299c32058253 . "./code.cg2js/cg-hotspot.fasl")
(#x7cee6a39311d7ce86e1f7aa2fd30579a . "./code.cg2js/cg-toolbar.fasl")
(#x7a96e990df07320af8d99998b21d00cb . "./code.cg2js/cg-clipboard-stack.fasl")
(#x7db64dd846c0758bcd361193f018c57d . "./code.cg2js/cg-tooltip.fasl")
(#x1a1e3f931253433f0a40e3f60fb3371c . "./code.cg2js/cg-calendar.fasl")
(#xaf48e7a3b15fc998499fbba039f56530 . "./code.cg2js/cg-item-list.fasl")
(#x87db93e2694b8569d6f6a2a93a9b83d2 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x68f7cd54a9a1d7a45a9b34453f1890e1 . "./code.cg2js/cg-trackbar.fasl")
(#x2b7f0fc6fee203e2f03f087a61760dbf . "./code.cg2js/cg-class-grid.fasl")
(#xba0112792839fe04bd422742615fd511 . "./code.cg2js/cg-dialog-item.fasl")
(#x61f54c562678dd5c277f7e7a09e74c2d . "./code.cg2js/cg-comtab.fasl")
(#xdc89685d1421f91d4cdb46243a3ece04 . "./code.cg2js/cg-lisp-text.fasl")
(#x68d0d0c6c3fec1674fd07249d11ed86a . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x19bd07f1bf9c22fcd41a9ebc9006e075 . "./code.cg2js/cg-button.fasl")
(#xd0bb384ecaf24c91d694c1b8cc47e9cc . "./code.cg2js/cg-printing.fasl")
(#x36758a386a739231885964a01b06aa51 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x62f2e392d879513357e3b4aa59535782 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x940b4fbc8fb2f70933c99d501842758f . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd2465e41443d38cc233e34d3dc580c01 . "./code.cg2js/cg-selected-object.fasl")
(#x5a1aee6e00aee26ae653a44bc5506f4f . "./code.cg2js/cg-text-or-combo.fasl")
(#x925b47bd7472324f74ff0e80eda8a956 . "./code.cg2js/cg-graphics-context.fasl")
(#x3f475ee445165f26f0613d0a487150de . "./code.cg2js/cg-edit-in-place.fasl")
(#x99e585060942127c0cd41234dd071fef . "./code.cg2js/cg-string-dialog.fasl")
(#x3fe8decca6a063edaaef33f5893cd9e7 . "./code.cg2js/cg-list-view.fasl")
(#x3eeb91bbf0a2915f0361d0663054c129 . "./code.cg2js/cg-intersect.fasl")
(#x648c5929b008ec24e4f23d20cc2b5002 . "./code.cg2js/cg-text-widget.fasl")
(#xc86903ff0ddbd1eccb73d9576ecf16c3 . "./code.cg2js/cg-directory-dialog.fasl")
(#xd515627e213b43ef4951eb13e984a466 . "./code.cg2js/cg-file-dialog.fasl")
(#x63f9f9ed742eace55fb3e55de1c3f708 . "./code.cg2js/cg-dde-utils.fasl")
(#x8dafad91b8de32336b53921cba13747d . "./code.cg2js/cg-nodes.fasl")
(#x2013ac958edc88fc549d028a2f6abb52 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xda0342307d6362c01398846790117556 . "./code.cg2js/cg-up-down-control.fasl")
(#x9197cdc3b6014fea88c3aace0e697a08 . "./code.cg2js/cg-get-position.fasl")
(#x4f629a29b18198ff3547640586abe953 . "./code.cg2js/cg-toggling-widget.fasl")
(#xf7e54099d371f95d74498d39d8e93c02 . "./code.cg2js/cg-acache.fasl")
(#x97c7919b2ed8560beb97d35bf443507c . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x02d1eb7394e786c20e7165170fdcf399 . "./code.cg2js/cg-base.fasl")
(#xeb41649dfaadbd876959009af949d488 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x1c631aae42dc234dd2136baa2794567f . "./code.cg2js/cg-find-string-dialog.fasl")
(#x9b10f9277599356b2fbc76a23ba96d45 . "./code.cg2js/cg-property.fasl")
(#x0457059514486f31f9a9660cf4dcec79 . "./code.cg2js/cg-editable-text.fasl")
(#xa9f9b16fa2db10f845dd658514ccc103 . "./code.cg2js/cg-lamp.fasl")
(#x34ffec2abe4688c4246e9412bc56a9b4 . "./code.cg2js/cg-progress-indicator.fasl")
(#x87d89c3b2aef1f57bffdf076968b5049 . "./code.cg2js/cg-chart-widget.fasl")
(#x2bb12bd970b2b29e8b4cee257e988b88 . "./code.cg2js/cg-common-control.fasl")
(#x49fd0a771ef518bb8bb2b97b22494171 . "./code.cg2js/cg-font-dialog.fasl")
(#x3f54f8b379446e4ac82b1993d48ce9ae . "./code.cg2js/cg-gesture-emulation.fasl")
(#x0600c4af8e7ebbd75f006858cbe8239f . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x67f1b35c9400c9d58ed72656222405ee . "./code.cg2js/cg-os-window.fasl")
(#x8ea0d59a34dfb11e5486f95a999f5e64 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xde688bce9b002c25986dd297367ac49e . "./code.cg2js/cg-pixmap.fasl")
(#x29422857eb46c220b19b77c19148dec4 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x68b8910ac09b0bcbdfd31d697013c388 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xaf0de89e019b25426252e5594241993b . "./code.cg2js/cg-scaling-stream.fasl")
(#x770096b0fa9e6425bf565d6c0796be99 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xdbabb69cd2079ebedc5d3f9b84b031ee . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x5c50b39550e1d2fdb71e7ebc8555d384 . "./code.cg2js/cg-timer.fasl")
(#x96330f7b93bf605cbee8f81a614fba31 . "./code.cg2js/cg-scroll-bar.fasl")
(#x9945f9e16442a05e12e6c4f3651e0668 . "./code.cg2js/cg-caret.fasl")
(#x488524146bca6e7c01e6e756e3000ffe . "./code.cg2js/cg-template-string.fasl")
(#xd8b8fa9df7ea88cbac7d9ae648a056eb . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xcb311fb2bac3e0306bd9390395cf2536 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xa8c96a4a8dfc3f8675b0bccf7cad36e8 . "./code.cg2js/cg-rich-edit.fasl")
(#xc986346be6b3b09871b5f37f401441d1 . "./code.cg2js/cg-combo-box.fasl")
(#xc45e63bb7070ae0be06c7ca95b5b447d . "./code.cg2js/cg-text-edit-pane.fasl")
(#xfdd31ce50673a2f62d726e174663d5ec . "./code.cg2js/cg-multi-picture-button.fasl")
(#xac1dc087350a353fa46f6836b4d63b35 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2716d8b90e6f0a3eefe6c91625dbe715 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xb82aab432b4c1eaf4aaa5b20919eb0d3 . "./code.cg2js/cg-outline.fasl")
(#x8ca401485c486514399345d34af09d26 . "./code.cg2js/cg-class-support.fasl")
(#xf5cbed40943df9abde7d2c498ca01757 . "./code.cg2js/cg-header-control.fasl")
(#x1d053e3b065ebe48a6e96ecc005479e3 . "./code.cg2js/cg-lettered-menu.fasl")
(#x4fc851e3710782968e3a6454358b686b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x9f6b951dccfa5317b46816b01e25bc9a . "./code.cg2js/cg-fill-texture.fasl")
(#xcdfd324ae93585d0db332cfc8803d6e8 . "./code.cg2js/cg-lisp-widget.fasl")
(#x770c4e57fe4410d714ac6afe07af5562 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x00768ec7843d433b7116186810ceb1c1 . "./code.cg2js/cg-clipboard.fasl")
(#x3272ef36af13493b1d74ee1f809eeac8 . "./code.cg2js/cg-wrap-string.fasl")
(#x84d349a342178c5659a133d47512aa1e . "./code.cg2js/cg-os-widget.fasl")
(#x5c52bff31e20f41155604c8622a3aa8d . "./code.cg2js/cg-check-box.fasl")
(#xfb373a63e4c454696c265bd054654091 . "./code.cg2js/cg-choice-list.fasl")
(#xd6974dbb70a06223d295d9bf6edec30a . "./code.cg2js/cg-layout.fasl")
(#x08084adb8bdbaf9641b85506d4d0b77a . "./code.cg2js/cg-group-box.fasl")
(#x1db8ee63e7a52346ba30754f1d715987 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xff4d9f936858e77828ce6b79a4d18d8a . "./code.cg2js/cg-radio-button.fasl")
(#x76f77b48b2645a359252a0ba5a3036d8 . "./code.cg2js/cg-tab-control.fasl")
(#x40af0f91feb2204ff9e7a117bec253c3 . "./code.cg2js/cg.fasl")
(#x203842eafd29903c97c53e4478fc0ac3 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xdd8979d2134384304da3ea0034565c7a . "./code.cg2js/cg-dropping-outline.fasl")
(#xf3635ea35cd20d9d512e05d5e390c37e . "./code.cg2js/cg-get-pixmap.fasl")
(#xf090b13029325b20d8789d4f5fffd520 . "./code.cg2js/cg-grid-widget.fasl")
(#xb4936eaecabd9a1770a345de927d0713 . "./code.cg2js/cg-utility-dialog.fasl")
(#x20d3c6430de1e06697ef0a0ff9a11bb8 . "./code.cg2js/cg-static-text.fasl")
(#xbe75d83d6ce90dbc6f1e3eff23bb1c3f . "./code.cg2js/cg-class-slot-grid.fasl")
(#xf4f8957302b7ff283745ec3fe57f976d . "./code.cg2js/cg-object-editor-layout.fasl")
(#x8969e794612d4147ec600817b97dd74f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x8339ab9cb68a99f2736c0bfe58f6f19a . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7333e988363547d739e011f65fcd4b76 . "./code.cg2js/cg-plot-widget.fasl")
(#x8c90702ae5adad0619ad4e071744271e . "./code.cg2js/cg-color-dialog.fasl")
(#x1bd4cb71b01f629bc44e66d4aa3525bd . "./code.cg2js/cg-icon-pixmap.fasl")
(#xc0c6452362821d4c0fe418aa412fd56d . "./code.cg2js/cg-icon.fasl")
(#x6726d06911aefee49dd7fd4533b7f711 . "./code.cg2js/cg-dev.fasl")
(#x5f4c753aa48904a9577af952d7304cca . "./code.cg2js/cg-choose-printer.fasl")
(#x680fddae8fa31a90ecc2d2dc4f2af0ed . "./code.cg2js/cg-intersect-posbox.fasl")
(#x138284ab37df40830b451fb628746bd5 . "./code.cg2js/cg-menu.fasl")
(#xba69c477ffffb5642abb6c173f64fe79 . "./code.cg2js/cg-yes-no-list.fasl")
(#xad7aa950e199c59ab112c662aac922a7 . "./code.cg2js/cg-palette.fasl")
(#x1ae366bff204a133b03c4c60d7180fc1 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x8dad29d33428cf2e6c289dc0e12385fd . "./code.cg2js/cg-message-dialog.fasl")
(#xebdd5cb9b66051b2971b1e14cda7379f . "./code.cg2js/cg-shortcut-menu.fasl")
(#x98c04d46e8f116b970b532b60da71a89 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xe02d848a6013e984b7772d51635aaa8f . "./code.cg2js/cg-split-bar.fasl")
(#x8cdb682c287d4cdf761a13b8345727d4 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x1275cfa51806bc30db00ddea3565c8ea . "./code.cg2js/cg-web-browser.fasl")
(#x1683d6e914dfff0c299b51e458b80e46 . "./code.cg2js/cg-project-window.fasl")
(#x176530626ab8fc3d886c8ec9feb15879 . "./code.cg2js/cg-picture-widget.fasl")
(#xdfb5aab49603b064c900c8bbada6787b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x90dcf8a6a623a6d1b1742eb2b499ddf1 . "./code.cg2js/cg-object-editor.fasl")
(#xeb2c89a41c6b37b3ca813e8829429b55 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x929ae96156bef03fe4f5580df2ad111a . "./code.cg2js/cg-paren-matching.fasl")
(#x80a5e8c010fa54d76de0bc155d04f688 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x0d694fc68a811b6cde79bedc7d4564bb . "./code.cg2js/cg-drawable.fasl")
(#x367269dddbee0d495215b5445420f324 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xdeb255759f877438c320f08429f71ed4 . "./code.cg2js/cg-html-widget.fasl")
(#x427e17eed81969796381361d77b3b087 . "./code.cg2js/cg-curve.fasl")
(#x270ea0c02aafd65f9cc2c81295e3c8f3 . "./code.cg2js/cg-yes-no-string.fasl")
(#x2a7a9a6666e94b83422adaf0dfb38f1f . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x21fa13efd54458ad7115373cdd39d5fa . "./code.cg2js/cg-html-dialog.fasl")
(#x44beccde0b0cd2501d5b18d70bb25dad . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x01278cfa267fc89c1ce6978bfc4ed5df . "./code.cg2js/cg-tray.fasl")
(#x872010f8030773f38f67536ffb88f070 . "./code.idejs/ide-apropos.fasl")
(#xaa83bd257bb27855da1e6fe0f4e661f3 . "./code.idejs/ide-window-list.fasl")
(#xffd7f1bb356f3d8a5d320ea01986bb5e . "./code.idejs/ide-class-browser.fasl")
(#x5942275c919cc32b3cdbc3f8d0c33c7a . "./code.idejs/ide-base.fasl")
(#xca2695728593bd187f09a6ac7804cab4 . "./code.idejs/ide-process-browser.fasl")
(#x51ae6bdbf37544ce5ac24cd31dfc0d8c . "./code.idejs/ide-acache.fasl")
(#xec87de373cb5b02ecb3e3b0e28801775 . "./code.idejs/ide-navigator.fasl")
(#x1c5778fb535a707c71d61673749d572a . "./code.idejs/ide-trace.fasl")
(#x69ae9fd3cf1b3094eeef737a9c7c114c . "./code.idejs/ide-project.fasl")
(#x3da9f1544a923b9149e6978e930be4d5 . "./code.idejs/ide-patch-dialog.fasl")
(#x0d25c87e229c0bc4f807df2c57f7b146 . "./code.idejs/ide-grapher.fasl")
(#x99f6690d2936f11212de5ce3c070ee40 . "./code.idejs/ide-dev.fasl")
(#x7633f2b83038167665661bda9f7d36d8 . "./code.idejs/ide-keyshort.fasl")
(#x62427467508ffcfa2080f90baeea1946 . "./code.idejs/ide-editor.fasl")
(#x680ead973ffb2073e30dc8c9e31e25e7 . "./code.idejs/ide-inspector.fasl")
(#x17ea3b455827eb77f2636876ee990485 . "./code.idejs/ide.fasl")
(#x1eb3ad3548ae17d3d4f77a6ab573d596 . "./code.idejs/ide-clipboard.fasl")
(#x90f1bf243e682ce993c9bec91064eafb . "./code.idejs/ide-find-definitions.fasl")
(#x7faceb155ff86116bcbd5613d58e1d6d . "./code.idejs/ide-form.fasl")
(#xf25043209b8f5cd278e59117988e9c89 . "./code.idejs/ide-intro.fasl")
(#x4ddb1ac7912a846cc0108a3e694e9817 . "./code.idejs/ide-find-in-files.fasl")
(#xcddbe46cab2fb07d95cbb035e9920b0c . "./code.idejs/ide-package-list.fasl")
(#x9857ad0a48deac6f6169f10fe3f49557 . "./code.idejs/ide-class-grapher.fasl")
(#xeb5b70ef6a4ced03becd92b82d3b0061 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
